package edu.lawrence.auction.entities;

import java.time.LocalDate;
import java.util.List;
import java.util.UUID;

import org.hibernate.annotations.JdbcTypeCode;
import org.hibernate.type.SqlTypes;

import edu.lawrence.auction.interfaces.dtos.AuctionDTO;
import jakarta.persistence.Column;
import jakarta.persistence.Entity;
import jakarta.persistence.GeneratedValue;
import jakarta.persistence.GenerationType;
import jakarta.persistence.Id;
import jakarta.persistence.JoinColumn;
import jakarta.persistence.ManyToOne;
import jakarta.persistence.OneToMany;
import jakarta.persistence.Table;

@Entity
@Table(name="auctions")
public class Auction {

	@Id
	@GeneratedValue(strategy = GenerationType.UUID)
	@Column(columnDefinition = "VARCHAR(45)")
	@JdbcTypeCode(SqlTypes.VARCHAR)
	private UUID auctionid;
	@ManyToOne
	@JoinColumn(name="seller")
	private User seller;
	private String item;
	private String description;
	private String imageurl;
	private int reserve;
	private LocalDate opens;
	private LocalDate closes;
	private boolean completed;
	@OneToMany(mappedBy="auction")
	List<Tag> tags;
	@OneToMany(mappedBy="auction")
	List<Bid> bids;
	
	public Auction() {}
	
	public Auction(AuctionDTO core) {
		item = core.getItem();
		description = core.getDescription();
		imageurl = core.getImageurl();
		opens = LocalDate.parse(core.getOpens());
		closes = LocalDate.parse(core.getCloses());
		completed = false;
	}

	public UUID getAuctionid() {
		return auctionid;
	}

	public void setAuctionid(UUID auctionid) {
		this.auctionid = auctionid;
	}

	public User getSeller() {
		return seller;
	}

	public void setSeller(User seller) {
		this.seller = seller;
	}

	public String getItem() {
		return item;
	}

	public void setItem(String item) {
		this.item = item;
	}

	public String getDescription() {
		return description;
	}

	public void setDescription(String description) {
		this.description = description;
	}

	public String getImageurl() {
		return imageurl;
	}

	public void setImageurl(String imageurl) {
		this.imageurl = imageurl;
	}

	public int getReserve() {
		return reserve;
	}

	public void setReserve(int reserve) {
		this.reserve = reserve;
	}

	public LocalDate getOpens() {
		return opens;
	}

	public void setOpens(LocalDate opens) {
		this.opens = opens;
	}

	public LocalDate getCloses() {
		return closes;
	}

	public void setCloses(LocalDate closes) {
		this.closes = closes;
	}

	public boolean getCompleted() {
		return completed;
	}

	public void setCompleted(boolean completed) {
		this.completed = completed;
	}

	public List<Tag> getTags() {
		return tags;
	}

	public void setTags(List<Tag> tags) {
		this.tags = tags;
	}
	
	public List<Bid> getBids() {
		return bids;
	}
	
	public void setBids(List<Bid> bids) {
		this.bids = bids;
	}
}
