package edu.lawrence.auction.interfaces;

import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import edu.lawrence.auction.entities.Auction;
import edu.lawrence.auction.entities.Profile;
import edu.lawrence.auction.entities.Shipping;
import edu.lawrence.auction.entities.User;
import edu.lawrence.auction.interfaces.dtos.Bill;
import edu.lawrence.auction.interfaces.dtos.Offer;
import edu.lawrence.auction.interfaces.dtos.ProfileDTO;
import edu.lawrence.auction.interfaces.dtos.Sale;
import edu.lawrence.auction.interfaces.dtos.ShippingDTO;
import edu.lawrence.auction.interfaces.dtos.UserDTO;
import edu.lawrence.auction.repositories.AuctionDAO;
import edu.lawrence.auction.repositories.UserDAO;


@RestController
@RequestMapping("/users")
@CrossOrigin(origins = "*")
public class UserController {

    private UserDAO dao;
    
    public UserController(UserDAO dao,AuctionDAO auctionDAO) {
        this.dao = dao;
    }

    @PostMapping("/login")
    public ResponseEntity<String> checkLogin(@RequestBody UserDTO user) {
        User result = dao.findByNameAndPassword(user.getName(), user.getPassword());
        if (result == null) {
            return ResponseEntity.status(HttpStatus.UNAUTHORIZED).body("Invalid user name or password");
        }
        return ResponseEntity.ok().body(result.getUserid().toString());
    }
    
    @PostMapping
    public ResponseEntity<String> save(@RequestBody UserDTO user) {
        if (user.getName().isBlank() || user.getPassword().isBlank()) {
            return ResponseEntity.status(HttpStatus.BAD_REQUEST).body("Empty user name or password");
        }

        String key = dao.save(user);
        if (key.equals("Duplicate")) {
            return ResponseEntity.status(HttpStatus.CONFLICT).body("User with this name already exists");
        } else if (key.equals("Error")) {
            return ResponseEntity.status(HttpStatus.INTERNAL_SERVER_ERROR).body("Can not generate key");
        }
        return ResponseEntity.ok().body(key);
    }
    
    @PostMapping("/{id}/profile")
    public ResponseEntity<String> saveProfile(@PathVariable UUID id,@RequestBody ProfileDTO profile) {
    	String result = dao.saveProfile(id,profile);
    	if(result.equals("Bad Id"))
    		return ResponseEntity.status(HttpStatus.UNAUTHORIZED).body("Invalid user id");
    	else if(result.equals("Duplicate"))
    		return ResponseEntity.status(HttpStatus.CONFLICT).body("Duplicate profile");
    	else if (result.equals("Error")) {
            return ResponseEntity.status(HttpStatus.INTERNAL_SERVER_ERROR).body("Can not store profile");
        }
    	return ResponseEntity.ok().body("Profile created");
    }
    
    @GetMapping("/{id}/profile")
    public ResponseEntity<ProfileDTO> getProfile(@PathVariable UUID id) {
    	Profile result = dao.findProfile(id);
    	if(result == null) {
    		return ResponseEntity.status(HttpStatus.NOT_FOUND).build();
    	}
    	ProfileDTO response = new ProfileDTO(result);
    	return ResponseEntity.ok().body(response);
    }
    
    @GetMapping("/{id}/auctions")
	public ResponseEntity<List<Auction>> getAuctions(@PathVariable UUID id) {
		List<Auction> results = dao.findAuctions(id);
		return ResponseEntity.ok().body(results);
	}
   
	@PostMapping("/{id}/shipping")
    public ResponseEntity<String> postShipping(@PathVariable UUID id,@RequestBody ShippingDTO shipping) {
    	
    	String result = dao.saveShipping(id,shipping);
    	if(result.equals("Bad Id"))
    		return ResponseEntity.status(HttpStatus.UNAUTHORIZED).body("Invalid user id");
    	else if (result.equals("Error")) {
            return ResponseEntity.status(HttpStatus.INTERNAL_SERVER_ERROR).body("Can not store shipping");
        }
    	return ResponseEntity.ok().body("Shipping saved.");
    }
    
    @GetMapping("/{id}/shipping")
    public ResponseEntity<List<ShippingDTO>> getShipping(@PathVariable UUID id) {
    	List<Shipping> results = dao.findShipping(id);
    	List<ShippingDTO> response = new ArrayList<ShippingDTO>();
    	for(Shipping s : results) {
    		response.add(new ShippingDTO(s));
    	}
    	return ResponseEntity.ok().body(response);
    }
    
    @GetMapping("/{id}/offers")
	public ResponseEntity<List<Offer>> getOffers(@PathVariable UUID id) {
		List<Offer> results = dao.findOffers(id);
		return ResponseEntity.ok().body(results);
	}
    
    @GetMapping("/{id}/accepted")
    public ResponseEntity<List<Sale>> getSales(@PathVariable UUID id) {
    	List<Sale> results = dao.findSales(id);
    	return ResponseEntity.ok().body(results);
    }
    
    @GetMapping("/{id}/billed")
    public ResponseEntity<List<Bill>> getBills(@PathVariable UUID id) {
    	List<Bill> results = dao.findBills(id);
    	return ResponseEntity.ok().body(results);
    }
    
    @GetMapping("/{id}/sold")
    public ResponseEntity<List<Bill>> getSold(@PathVariable UUID id) {
    	List<Bill> results = dao.findSoldBills(id);
    	return ResponseEntity.ok().body(results);
    }
    
    @GetMapping("/{id}/shipped")
    public ResponseEntity<List<Bill>> getShipped(@PathVariable UUID id) {
    	List<Bill> results = dao.findShippedBills(id);
    	return ResponseEntity.ok().body(results);
    }
}