package edu.lawrence.auction.repositories;

import java.time.LocalDate;
import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import edu.lawrence.auction.entities.Auction;
import edu.lawrence.auction.entities.Bid;
import edu.lawrence.auction.entities.Tag;
import edu.lawrence.auction.entities.User;
import edu.lawrence.auction.interfaces.dtos.AuctionDTO;
import edu.lawrence.auction.interfaces.dtos.BidDTO;
import jakarta.persistence.EntityManager;
import jakarta.transaction.Transactional;

@Repository
public class AuctionDAO {
	@Autowired
	EntityManager em;
	
	@Transactional
	public String save(AuctionDTO auction) {
		User user = em.find(User.class,UUID.fromString(auction.getSeller()));
		if(user == null) 
			return "Bad Id";
		
		Auction newAuction = new Auction(auction);
		newAuction.setSeller(user);
		em.persist(newAuction);
		
		for(String t : auction.getTags()) {
			Tag newTag = new Tag();
			newTag.setAuction(newAuction);
			newTag.setTag(t);
			em.persist(newTag);
		}
		return newAuction.getAuctionid().toString();
	}
	
	@Transactional
	public Auction findById(UUID id) {
		return em.find(Auction.class,id);
	}
	
	@Transactional
	public List<Auction> findActive() {
		return em.createQuery("select a from Auction a where opens<=:now and closes>=:now",Auction.class)
				.setParameter("now", LocalDate.now()).getResultList();
	}
	
	@Transactional
	public List<Auction> findActiveByTag(String tag) {
		List<Auction> results = new ArrayList<Auction>();
		List<Tag> tags = em.createQuery("select t from Tag t where t.tag=:tag and t.auction.opens<=:now and t.auction.closes>=:now",Tag.class)
				.setParameter("tag", tag).setParameter("now", LocalDate.now()).getResultList();
		for(Tag t : tags) {
			results.add(t.getAuction());
		}
		return results;
	}
	
	@Transactional
	public String saveBid(UUID auctionid,BidDTO bid) {
		Auction auction = em.find(Auction.class, auctionid);
		if(auction == null)
			return "Bad Auction Id";
		
		User user = em.find(User.class, UUID.fromString(bid.getBidder()));
		if(user == null)
			return "Bad User Id";
		
		Bid newBid = new Bid(bid);
		newBid.setAuction(auction);
		newBid.setBidder(user);
		em.persist(newBid);
		return "Created";
	}
}
