package edu.lawrence.auction.repositories;

import java.time.LocalDate;
import java.util.List;
import java.util.UUID;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import edu.lawrence.auction.entities.Auction;
import edu.lawrence.auction.entities.Bid;
import edu.lawrence.auction.entities.Purchase;
import edu.lawrence.auction.entities.Shipping;
import edu.lawrence.auction.interfaces.dtos.BillResponse;
import edu.lawrence.auction.interfaces.dtos.OfferResponse;
import edu.lawrence.auction.interfaces.dtos.ShippingCharge;
import edu.lawrence.auction.interfaces.dtos.ShippingConfirmation;
import jakarta.persistence.EntityManager;
import jakarta.transaction.Transactional;

@Repository
public class PurchaseDAO {
	@Autowired
	EntityManager em;

	@Transactional
	public boolean saveOfferResponse(OfferResponse or) {
		Purchase p = em.find(Purchase.class, UUID.fromString(or.getPurchaseid()));
		if(p == null)
			return false;
		if(or.getAccept()) {
			Shipping s = em.find(Shipping.class, or.getShippingid());
			p.setShipping(s);
			p.setStatus("Confirmed");
			return true;
		} else {
			Bid b = p.getBid();
			em.remove(b);
			em.remove(p);
			return false;
		}
	}
    
	@Transactional
	public void saveShipping(ShippingCharge sc) {
    	Purchase p = em.find(Purchase.class, UUID.fromString(sc.getPurchaseid()));
		if(p != null) {
			p.setCharged(sc.getCharge());
			p.setStatus("Charged");
		}
    }
    
	@Transactional
	public boolean saveBillResponse(BillResponse br) {
    	Purchase p = em.find(Purchase.class, UUID.fromString(br.getPurchaseid()));
		if(p == null)
			return false;
    	if(br.getAccepted()) {
    		p.setStatus("Paid");
    		return true;
    	} else {
    		Bid b = p.getBid();
			em.remove(b);
			em.remove(p);
			return false;
    	}
    }
    
	@Transactional
	public void saveShipConfirm(ShippingConfirmation sc) {
    	Purchase p = em.find(Purchase.class, UUID.fromString(sc.getPurchaseid()));
		if(p != null) {
			p.setTracking(sc.getTracking());
			p.setStatus("Shipped");
		}
    }
    
	@Transactional
	public void concludeAuctions() {
    	// Find all of the completed auctions for which there is no purchase
    	List<Auction> auctions =  em.createQuery("select a from Auction a where closes<:now and completed=false",Auction.class)
				.setParameter("now", LocalDate.now()).getResultList();

    	for(Auction a : auctions) {
    		// Find the winning bid
    		List<Bid> bids = a.getBids();
    		if(bids.size() > 0) {
	    		Bid winner = bids.get(0);
	    		for(Bid b : bids) {
	    			if(b.getBid() > winner.getBid())
	    				winner = b;
	    		}
	    		// Create a Purchase
	    		Purchase p = new Purchase();
	    		p.setBid(winner);
	    		p.setStatus("Won_bid");
	    		em.persist(p);
    		}
    		// Mark the auction as completed
	    	a.setCompleted(true);
    		
    	}
    }
}
